@@ -39,7 +39,8 @@
   #:use-module (dzn vm evaluate)
   #:use-module (dzn vm goops)
   #:use-module (dzn vm runtime)
-  #:export (%debug
+  #:export (%%debug
+            %debug
             %debug?
             %next-input
             %startup-info
@@ -149,16 +150,87 @@
 ;;;
 ;;; Code:
 
-;;; Debug facility
+;;; Should debug info be printed?
 (define %debug? (make-parameter #f))
 
-(define-syntax-rule (%debug fmt arg ...)
-  (when (%debug?)
-    (format (current-error-port) fmt arg ...)))
-
 ;; Is the input trail to be matched exactly?
 (define %strict? (make-parameter #f))
 
+
+;;;
+;;; Debug facility.
+;;;
+(define-method (%debug (fmt <string>) . args)
+  (when (%debug?)
+    (apply format (current-error-port) fmt args)
+    (newline (current-error-port))))
+
+(define-method (%debug (location <pair>) (fmt <string>) . args)
+  (when (%debug?)
+   (format (current-error-port) "~a:~a:~a: ~a\n"
+           (assoc-ref location 'filename)
+           (1+ (assoc-ref location 'line))
+           (assoc-ref location 'column)
+           (apply format #f fmt args))))
+
+(define-method (%debug (location <pair>) (pc <program-counter>) (fmt <string>) . args)
+  (apply %debug location fmt args)
+  (when (and (%debug?) (> (%debug?) 1))
+    (%debug pc)))
+
+(define-method (%debug-statement (pc <program-counter>) (o <top>))
+  "*rtc*")
+
+(define-method (%debug-statement (pc <program-counter>) (o <statement>))
+  (name o))
+
+(define-method (%debug-statement (pc <program-counter>) (o <assign>))
+  (format #f "~a ~a=~a" (next-method)
+          (.variable.name o)
+          (->sexp (eval-expression pc (.expression o)))))
+
+(define-method (%debug-statement (pc <program-counter>) (o <variable>))
+  (format #f "~a ~a=~a" (next-method)
+          (.name o)
+          (->sexp (eval-expression pc (.expression o)))))
+
+(define-method (%debug-statement (pc <program-counter>) (o <guard>))
+  (format #f "~a ~a=>~a" (next-method)
+          (->sexp (.expression o))
+          (true? (eval-expression pc (.expression o)))))
+
+(define-method (%debug-statement (pc <program-counter>) (o <on>))
+  (format #f "~a ~a" (next-method) (map trigger->string (ast:trigger* o))))
+
+(define-method (%debug (pc <program-counter>))
+  (when (%debug?)
+    (let* ((statement (.statement pc))
+           (location (and=> statement .location))
+           (file-name (and=> location .file-name))
+           (line (and=> location .line))
+           (column (and=> location .column))
+           (instance (.instance pc))
+           (state (.state pc)))
+      (format (current-error-port)
+              "~a:~a:~a:[~a] [~a~a] ~a\n"
+              file-name line column  (name instance)
+              (if (and=> (.trigger pc) (is? <q-trigger>)) "<q> "
+                  "")
+              (and=> (.trigger pc) trigger->string)
+              (%debug-statement pc statement))
+      (when (> (%debug?) 1)
+        (format (current-error-port)
+                "~a\n"
+                state))
+      (when (> (%debug?) 2)
+        (format (current-error-port) "~a\n" pc))
+      (unless (= (%debug?) 1)
+        (newline (current-error-port))))))
+
+(define (%%debug . rest)
+  (when (and (%debug?) (> (%debug?) 1))
+    (apply %debug rest)))
+
 
 ;;;
 ;;; Input, labels
@@ -176,6 +248,7 @@
       ((h t ...) o)
       (_ (format #f "~a" o))))
   (let* ((trail (string-join (string-split trail #\,) " "))
+         (trail (string-join (string-split trail #\;) " "))
          (trail (with-input-from-string trail read-input-file))
          (trail (map event->string trail))
          (loop-index (list-index (cute equal? <> "<loop>") trail))
@@ -660,7 +733,8 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
          (pcs (filter (compose (cute eq? <> instance) .instance) pcs))
          (triggers (filter-map .trigger pcs)))
     (and (pair? triggers)
-         (last triggers))))
+         (or (find (is? <q-trigger>) triggers)
+             (last triggers)))))
 
 
 ;;;
@@ -680,10 +754,9 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                #:defer (.defer pc)
                #:external-q (.external-q pc)
                #:released (.released pc)
-               #:skip-compliance? (.skip-compliance? pc)
                #:state (.state pc)
                #:trail (.trail pc))))
-    (%debug "  ~s ~s <block> ~a [~a] => [~a]\n"
+    (%debug (current-source-location) "  ~s ~s <block> ~a [~a] => [~a]"
             (name instance)
             (and=> (.trigger pc) trigger->string)
             (runtime:instance->string port)
@@ -777,8 +850,8 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
      (let ((instance (.instance blocked-pc)))
        (if (is-a? instance <runtime:component>) pc
            (begin
-             (%debug
-              "  ~s ~s <switch-context blocked-on-boundary> ~a [~a] => [~a]\n"
+             (%debug (current-source-location)
+              "  ~s ~s <switch-context blocked-on-boundary> ~a [~a] => [~a]"
               (name instance)
               (and=> (.trigger blocked-pc) trigger->string)
               (runtime:instance->string port)
@@ -830,7 +903,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                #:running-defer? (.running-defer? pc)
                #:state (.state pc)
                #:trail (.trail pc))))
-    (%debug "  ~s ~s <collateral-block> ~a [~a] => [~a]\n"
+    (%debug (current-source-location) "  ~s ~s <collateral-block> ~a [~a] => [~a]"
             ((compose name .instance) pc)
             (and=> (.trigger pc) trigger->string)
             (runtime:instance->string blocked-port)
@@ -857,7 +930,8 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                (collateral-released (if blocked? collateral-released
                                         (delete r:port-collateral collateral-released)))
                (trigger (.trigger released-pc)))
-          (%debug "  ~s ~s <switch-context ~a> ~a [~a] => [~a]\n"
+          (%debug (current-source-location)
+                  "  ~s ~s <switch-context ~a> ~a [~a] => [~a]"
                   (name instance)
                   (and=> trigger trigger->string)
                   (if blocked? "block" "collateral")
@@ -887,7 +961,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 ;;;
 
 (define-method (enqueue (pc <program-counter>) (ast <ast>) (instance <runtime:component>) (trigger <trigger>))
-  (%debug "  ~s ~s ~a => ~s\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) "<enqueue>" (trigger->string trigger))
+  (%debug (current-source-location) "*enqueue*")
   (let* ((state (get-state pc instance))
          (q (.q state)))
     (if (= (length q) (%queue-size))
@@ -897,15 +971,15 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
         (set-deferred (set-state pc (clone state #:q (append q (list trigger)))) instance))))
 
 (define-method (dequeue (pc <program-counter>))
+  (%debug (current-source-location) "*dequeue*")
   (let* ((state (get-state pc))
          (q (.q state))
          (pc (set-state pc (clone state #:q (cdr q))))
          (trigger (car q)))
-    (%debug "  ~s ~s ~a => ~s\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) "<dequeue>" (trigger->string trigger))
     (values pc trigger)))
 
 (define-method (enqueue-external (pc <program-counter>) (ast <ast>) (trigger <trigger>))
-  (%debug "  ~s ~s ~a => ~s\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) "<enqueue-external>" (trigger->string trigger))
+  (%debug (current-source-location) "*external-enqueue*")
   (let* ((external-q (.external-q pc))
          (instance (.instance pc))
          (q (or (assoc-ref external-q instance) '())))
@@ -922,6 +996,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
           (clone pc #:external-q external-q)))))
 
 (define-method (dequeue-external (pc <program-counter>) (instance <runtime:port>))
+  (%debug (current-source-location) "*external-dequeue*")
   (let* ((external-q (.external-q pc))
          (q (assoc-ref external-q instance)))
     (if (or (not q) (null? q)) (values pc #f)
@@ -931,7 +1006,6 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                                (acons instance (cdr q) external-q)))
                (pc (clone pc #:external-q external-q))
                (trigger (car q)))
-          (%debug "  ~s ~s ~a => ~s\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) "<dequeue-external>" (trigger->string trigger))
           (values pc trigger)))))
 
 (define-method (get-handling (pc <program-counter>) (instance <runtime:instance>))
@@ -973,7 +1047,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
     (clone pc #:defer defer)))
 
 (define-method (flush (pc <program-counter>) instance)
-  (%debug "  ~s ~s ~a\n" (name instance) (and=> (.trigger pc) trigger->string) "<flush>")
+  (%debug (current-source-location) pc "*flush*")
   (let* ((orig-pc pc)
          (trigger (and=> (as (.q (get-state pc instance)) <pair>)
                          car))
@@ -993,7 +1067,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                    (get-handling pc deferred))
                orig-pc)
               (else
-               (%debug "  flush deferred: ~s\n" deferred)
+               (%debug (current-source-location) "  flush deferred: ~s" deferred)
                (flush pc deferred))))))))
 
 (define-method (flush (pc <program-counter>))
@@ -1337,13 +1411,14 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
          (variables (if (null? q) variables
                         (cons (string-append "q=" q-string) variables))))
     (and (not (equal? path '("client")))
-         (or (pair? variables) (pair? q))
+         (or (pair? variables) (pair? q) (.handling o))
          (string-append
           (string-join path ".")
           (if (pair? path) ":" "")
           "["
           (string-join variables ",\n")
-          "]"))))
+          "]"
+          (if (.handling o) "\nhandling" "")))))
 
 (define-method (state->string (o <system-state>))
   (let ((state-list (.state-list o)))
@@ -1368,7 +1443,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
      "<deadlock>")
     (_
      (string-join
-      (cons (state->string (.state o) (not (.skip-compliance? o)))
+      (cons (state->string (.state o) #t)
             (append
              (let* ((instance (.instance o))
                     (statement (.statement o))
@@ -1431,7 +1506,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                  `("blocked-on-boundary:"
                    ,@(map trigger->string (blocked-on-boundary-statements o))))
              (if (null? (.external-q o)) '()
-                 (list (external-q->string (.external-q o))))
+                 (list "ext-q:" (external-q->string (.external-q o))))
              (let ((defer (.defer o)))
                (if (null? defer) '()
                    `("defer:"
